import datetime as dt
from pathlib import Path
import json

import obspython as obs
from playsound import playsound

### Constants ###

REPLAY_SAVED_SOUND_PATH = Path(__file__).parent.resolve() / "buffer-saved.wav"
BUFFER_READY_SOUND_PATH = Path(__file__).parent.resolve() / "buffer-ready.wav"
START_STOP_DELAY = 200
INIT_DELAY = 500


### Global variables ###

settings = {}
restarting_replay_buffer = False
buffer_start_attempt = 1


### Utility functions ###


def log(message):
    print(f"[{dt.datetime.now().isoformat(sep=' ', timespec='seconds')}] {message}")


def decode_settings(obs_settings) -> dict:
    return json.loads(obs.obs_data_get_json(obs_settings))


### Timer functions ###


def stop_replay_buffer():
    log("Stopping replay buffer...")
    obs.timer_remove(stop_replay_buffer)
    obs.obs_frontend_replay_buffer_stop()


def start_replay_buffer():
    global buffer_start_attempt
    log(f"Starting replay buffer... (attempt {buffer_start_attempt})")
    obs.obs_frontend_replay_buffer_start()
    buffer_start_attempt += 1


### Event callbacks ###


def on_replay_buffer_saved():
    global restarting_replay_buffer

    if restarting_replay_buffer:
        return

    log("Replay saved")

    if settings["play_sound_save"]:
        playsound(REPLAY_SAVED_SOUND_PATH.as_posix())

    if settings["clear_buffer"]:
        restarting_replay_buffer = True
        obs.timer_add(stop_replay_buffer, START_STOP_DELAY)


def on_replay_buffer_stopped():
    if restarting_replay_buffer:
        obs.timer_add(start_replay_buffer, START_STOP_DELAY)


def on_replay_buffer_started():
    global restarting_replay_buffer, buffer_start_attempt
    if restarting_replay_buffer:
        log("Replay buffer started")
        obs.timer_remove(start_replay_buffer)
        if settings["play_sound_ready"]:
            playsound(BUFFER_READY_SOUND_PATH.as_posix())
        restarting_replay_buffer = False
        buffer_start_attempt = 1


def on_event(event):
    match event:
        case obs.OBS_FRONTEND_EVENT_REPLAY_BUFFER_SAVED:
            on_replay_buffer_saved()
        case obs.OBS_FRONTEND_EVENT_REPLAY_BUFFER_STOPPED:
            on_replay_buffer_stopped()
        case obs.OBS_FRONTEND_EVENT_REPLAY_BUFFER_STARTED:
            on_replay_buffer_started()


### Script configuration ###


def script_load(obs_settings):
    global settings, restarting_replay_buffer
    settings.update(decode_settings(obs_settings))
    obs.obs_frontend_add_event_callback(on_event)
    log("Script loaded successfully")
    if not obs.obs_frontend_replay_buffer_active():
        restarting_replay_buffer = True
        obs.timer_add(start_replay_buffer, INIT_DELAY)


def script_update(obs_settings):
    global settings
    settings.update(decode_settings(obs_settings))
    log(f"Config: {settings}")


def script_properties():
    props = obs.obs_properties_create()
    obs.obs_properties_add_bool(props, "play_sound_save", "Play sound on save")
    obs.obs_properties_add_bool(props, "play_sound_ready", "Play sound on ready")
    obs.obs_properties_add_bool(props, "clear_buffer", "Prevent overlaps")
    return props


def script_defaults(obs_settings):
    global settings
    obs.obs_data_set_default_bool(obs_settings, "play_sound_save", True)
    obs.obs_data_set_default_bool(obs_settings, "play_sound_ready", True)
    obs.obs_data_set_default_bool(obs_settings, "clear_buffer", True)
    settings.update(decode_settings(obs.obs_data_get_defaults(obs_settings)))
